!---------------------------------------------------------------------------------------
! PREP_FLUXES: convertgrid
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> convertgrid
!! 
!! Purpose:    Interpolates or averages data from a given input grid to a given 
!!             output grid 
!!
!! Interface:  
!!
!!    Inputs   
!!             midpoints -  logical (true: coords are midpoints)
!!             lon_in    -  longitude of input grid
!!             lat_in    -  latitude of input grid
!!             flx_tmp   -  data on input grid
!!             lon_out   -  longitude of output grid
!!             lat_out   -  latitude of output grid
!!             flx_out   -  data on output grid (zero matrix on input)
!!
!!    Externals
!!             sort
!!             gridarea
!!
!---------------------------------------------------------------------------------------

subroutine convertgrid(midpoints, lon_in, lat_in, flx_tmp, &
                          lon_out, lat_out, flx_out)

  use mod_var

  implicit none

  logical,                                  intent(in)     :: midpoints
  real, dimension(nx_in,ny_in,ntime_out),   intent(in out) :: flx_tmp
  real, dimension(nx_in),                   intent(in out) :: lon_in
  real, dimension(ny_in),                   intent(in out) :: lat_in
  real, dimension(nx_out),                  intent(in out) :: lon_out
  real, dimension(ny_out),                  intent(in out) :: lat_out
  real, dimension(nx_out,ny_out,ntime_out), intent(in out) :: flx_out
  
  integer :: nt, ix, jy, ix1, ix2, jy1, jy2, ii, jj
  real    :: xl, xr, xm, yl, yr, ym
  real    :: xolap1, xolap2, yolap1, yolap2
  real    :: mass, area, aolap, aolaptot 
  real    :: tot_out, tot_in

  integer, dimension(nx_in)      :: indx
  integer, dimension(ny_in)      :: indy
  real, dimension(nx_in)         :: dx_in
  real, dimension(ny_in)         :: dy_in
  real, dimension(nx_in)         :: lono
  real, dimension(ny_in)         :: lato
  real, dimension(nx_in)         :: wlon_in
  real, dimension(ny_in)         :: slat_in
  real, dimension(nx_out)        :: wlon_out
  real, dimension(ny_out)        :: slat_out
  real, dimension(nx_in,ny_in)   :: area_xy

  tot_in = 0.
  tot_out = 0.
  dx_in(:) = 0.
  dy_in(:) = 0.
  wlon_in(:) = 0.
  slat_in(:) = 0.
  wlon_out(:) = 0.
  slat_out(:) = 0.
  flx_out(:,:,:) = 0.

  ! check lon and lat dimensions
  lato = lat_in
  call sort(ny_in, lato)
  ! indices to lat
  do jy = 1, ny_in
    indy(jy) = minloc(lato,dim=1,mask=lato.eq.lat_in(jy))
  end do
  lat_in = lato
  where (lon_in.gt.180.) lon_in = lon_in - 360.
  lono = lon_in
  call sort(nx_in, lono)
  ! indices to lon
  do ix = 1, nx_in
    indx(ix) = minloc(lono,dim=1,mask=lono.eq.lon_in(ix))
  end do
  lon_in = lono
  ! reorder flx
  flx_tmp(:,:,:) = flx_tmp(indx,indy,:)

  ! input resolution
  do ix = 2, nx_in
    dx_in(ix) = abs(lon_in(ix)-lon_in(ix-1))
  end do
  dx_in(1) = dx_in(2)
  do jy = 2, ny_in
    dy_in(jy) = abs(lat_in(jy)-lat_in(jy-1))
  end do
  dy_in(1) = dy_in(2)
  
  ! round dx and dy to 7dp
  dx_in = anint(dx_in*1.e7)/1.e7
  dy_in = anint(dy_in*1.e7)/1.e7

  ! if not midpoints check if west and south boundaries given
  if(.not.midpoints) then
    if((lon_in(1)-dx_in(1)).le.-180.) then
      ! looks like global field
      if(lon_in(1).gt.-180.) then
        ! looks like east boundary 
        lon_in = lon_in - dx_in
      endif
    endif
    if((lat_in(1)-dy_in(1)).le.-90.) then
      ! looks like global field
      if(lat_in(1).gt.-90.) then
        ! looks like north boundary 
        lat_in = lat_in - dy_in
      endif
    endif
  endif


  print*, 'lat_in:', lat_in
  print*, 'lon_in:', lon_in

  if(midpoints) then
    ! western edges
    do ix = 1, nx_in
      wlon_in(ix) = lon_in(ix)-0.5*dx_in(ix)
    end do
    ! southern edges
    do jy = 1, ny_in
      slat_in(jy) = lat_in(jy)-0.5*dy_in(jy)
    end do
  else
    wlon_in = lon_in
    ! longitude midpoints
    do ix = 1, nx_in
      lon_in(ix) = lon_in(ix)+0.5*dx_in(ix)
    end do
    slat_in = lat_in
    ! latitude midpoints
    do jy = 1, ny_in
      lat_in(jy)=lat_in(jy)+0.5*dy_in(jy)
    end do
  endif  

  if( (nx_in.eq.nx_out) .and. (ny_in.eq.ny_out) ) then
    write(*,*) 'Comment: input and output grids are the same'
    wlon_out = wlon_in
    slat_out = slat_in
    flx_out = flx_tmp
    go to 20
  endif

  ! loop over time and then y and x dimensions
  do nt = 1, ntime_out
    do jy = 1, ny_out
      slat_out(jy) = lly_out+float(jy-1)*dy_out
      do ix = 1, nx_out
        wlon_out(ix) = llx_out+float(ix-1)*dx_out
        ix1 = maxloc(wlon_in,dim=1,mask=wlon_in.le.wlon_out(ix))
        ix2 = maxloc(wlon_in,dim=1,mask=wlon_in.lt.(wlon_out(ix)+dx_out))
        jy1 = maxloc(slat_in,dim=1,mask=slat_in.le.slat_out(jy))
        jy2 = maxloc(slat_in,dim=1,mask=slat_in.lt.(slat_out(jy)+dy_out))
        if( (ix1.eq.0) .or. (ix2.eq.0) .or. (jy1.eq.0) .or. (jy2.eq.0) ) then
          write(*,*) 'Warning: one of ix1, ix2, jy1, jy2 is zero'
          go to 10
        endif
        mass = 0.
        aolaptot = 0.
        ! loop over grid cells in input grid which overlap with the grid cell in output grid
        do jj = jy1, jy2
          do ii = ix1, ix2
            ! overlapping area has boundaries xolap1,xolap2 and yolap1,yolap2
            xolap1 = max(wlon_in(ii),wlon_out(ix))
            xolap2 = min(wlon_in(ii)+dx_in(ii),wlon_out(ix)+dx_out)
            yolap1 = max(slat_in(jj),slat_out(jy))
            yolap2 = min(slat_in(jj)+dy_in(jj),slat_out(jy)+dy_out)
            ! calculate overlapping area
            call gridarea((yolap1+yolap2)/2., abs(yolap2-yolap1), abs(xolap2-xolap1), aolap)
            mass = mass + aolap*flx_tmp(ii,jj,nt)
            aolaptot = aolaptot + aolap    
          end do
        end do
        if(aolaptot.ne.0) then
          ! area flux (kg/m2/h)
          flx_out(ix,jy,nt) = mass/aolaptot
        endif
        tot_out = tot_out+mass
        ! complete righthand edge of grid
        mass = 0.
        aolaptot = 0.
        if(ix.eq.nx_out) then
          xolap1 = wlon_in(ix2)
          xolap2 = wlon_out(ix)+dx_out
          do jj = jy1, jy2
            yolap1 = max(slat_in(jj),slat_out(jy))
            yolap2 = min(slat_in(jj)+dy_in(jj),slat_out(jy)+dy_out)
            ! calculate overlapping area
            call gridarea((yolap1+yolap2)/2., abs(yolap2-yolap1), abs(xolap2-xolap1), aolap)
            mass = mass + aolap*flx_tmp(ix2,jj,nt)
            aolaptot = aolaptot + aolap
          end do
          if(aolaptot.ne.0) then
            flx_out(ix,jy,nt) = mass/aolaptot
          endif   
          tot_out = tot_out + mass
        endif
        ! complete upper edge of grid
        mass = 0.
        aolaptot = 0.
        if(jy.eq.ny_out) then
          yolap1 = slat_in(jy2)
          yolap2 = slat_out(jy)+dy_out
          do ii = ix1, ix2
            xolap1 = max(wlon_in(ii),wlon_out(ix))
            xolap2 = min(wlon_in(ii)+dx_in(ii),wlon_out(ix)+dx_out)
            ! calculate overlapping area
            call gridarea((yolap1+yolap2)/2., abs(yolap2-yolap1), abs(xolap2-xolap1), aolap)
            mass = mass + aolap*flx_tmp(ii,jy2,nt)
            aolaptot = aolaptot + aolap
          end do
          if(aolaptot.ne.0) then
            flx_out(ix,jy,nt) = mass/aolaptot
          endif
          tot_out = tot_out + mass
        endif
        ! check for undefined values
        if(isnan(flx_out(ix,jy,nt))) flx_out(ix,jy,nt) = 0.
10      continue
      end do ! nx_out
    end do ! ny_out
  end do ! ntime

20 continue

  ! check total in versus total out
  ix1 = maxloc(wlon_in,dim=1,mask=wlon_in.le.llx_out)
  ix2 = minloc(wlon_in,dim=1,mask=wlon_in.ge.(llx_out+nx_out*dx_out-dx_in(nx_in)))
  jy1 = maxloc(slat_in,dim=1,mask=slat_in.le.lly_out)
  jy2 = minloc(slat_in,dim=1,mask=slat_in.ge.(lly_out+ny_out*dy_out-dy_in(ny_in)))
  
  print*, 'ix1, ix2, jy1, jy2: ',ix1,ix2,jy1,jy2

  ! calculate input grid area
  area_xy(:,:) = 0.
  do jy = jy1, jy2
    do ix = ix1, ix2
      call gridarea(lat_in(jy), dy_in(jy), dx_in(ix), area)
      area_xy(ix,jy) = area
    end do
  end do
  
  ! calculate total in
  do nt = 1, ntime_out
    tot_in = tot_in + sum( sum(flx_tmp(:,:,nt)*area_xy(:,:),dim=1) )
  end do

  print*, tot_in*24.*365./float(ntime_out)/1.e9
  print*, tot_out*24.*365./float(ntime_out)/1.e9

  write(*,fmt='(1x,''Total mass in:'',1x,F10.4,1x,''Tg/y'')') tot_in*24.*365./float(ntime_out)/1.e9
  write(*,fmt='(1x,''Total mass out:'',1x,F10.4,1x,''Tg/y'')') tot_out*24.*365./float(ntime_out)/1.e9

  do ix = 1, nx_out
    lon_out(ix) = wlon_out(ix)+dx_out*0.5
  end do
  do jy = 1, ny_out
    lat_out(jy) = slat_out(jy)+dy_out*0.5
  end do


end subroutine convertgrid


