!---------------------------------------------------------------------------------------
! FLEXINVERT: get_initconc
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> get_initconc
!!
!! Purpose:    Reads the initial concentration files and interpolates these to
!!             the flexpart grid
!!
!! Interface:
!!
!!    Inputs
!!             files     -  file data structure
!!             config    -  configuration settings data structure
!!             filename  -  name of 4D concentration file
!!             ntime     -  time dimension of concentration field 
!!             concini   -  concentrations on flexpart grid (zero array on input)
!! 
!!    Externals
!!             convertgrid
!!             interpz
!!             check
!!             
!! To do:
!!
!!    Add temperature to EGG4 and CAMS-N2O files for more accurate geopotential
!!    heights
!!
!---------------------------------------------------------------------------------------

subroutine get_initconc(files, config, filename, ntime, concini)

  use mod_var
  use mod_settings
  use mod_tools,   only : sort
  use mod_save,    only : check
  use netcdf
  
  implicit none

  type (files_t),                              intent(in)     :: files
  type (config_t),                             intent(in)     :: config
  character(len=max_path_len),                 intent(in)     :: filename
  integer,                                     intent(in)     :: ntime
  real, dimension(nxgrid,nygrid,nzgrid,ntime), intent(in out) :: concini

  character(len=max_name_len)                                 :: lonname, latname, varname, prsname, timename
  logical                                                     :: midpoints
  integer                                                     :: ncid, xid, yid, zid, tid, varid
  integer                                                     :: n, kz, status, iproduct, zkind
  integer                                                     :: nx, ny, nz, nt  
  integer                                                     :: nx_out, ny_out
  real, dimension(:), allocatable                             :: lon, lat
  real(kind=8), dimension(:), allocatable                     :: lono, lato, alto
  real, dimension(:), allocatable                             :: ap, bp, mpres
  integer, dimension(:), allocatable                          :: indx, indy, indz
  real, dimension(nxgrid)                                     :: lon_out
  real, dimension(nygrid)                                     :: lat_out
  real, dimension(:,:), allocatable                           :: rpres, scalh
  real, dimension(:,:,:), allocatable                         :: work1, work2, work3, work4
  real, dimension(:,:,:), allocatable                         :: geoh, geoho, pres, surfp, pw
  real, dimension(:,:,:,:), allocatable                       :: conc, conco, humid, temp
  real                                                        :: sclfact, offset

  ! initialize
  ! ----------

  ! don't pass globals to subroutines
  lon_out = gbl_lon
  lat_out = gbl_lat
  nx_out = nxgrid
  ny_out = nygrid

  ! which initial concentration field product
  ! 1 = CAMS CH4, 2 = EGG4, 3 = CAMS N2O, 4 = FP-CTM, 5 = NOAA interpolated, 6 = FP-CTM (monthly)
  iproduct = int(files%init_type)
  varname = files%varname_init

  select case ( iproduct)
    case (1)
    ! CAMS CH4
      lonname = 'longitude'
      latname = 'latitude'
      timename = 'time'
      prsname = 'level'
    case (2)
    ! EGG4
      lonname = 'longitude'
      latname = 'latitude'
      timename = 'time'
      prsname = 'level'
    case (3)
    ! CAMS N2O
      lonname = 'longitude'
      latname = 'latitude'
      timename = 'time'
      prsname = 'level'
    case (4)
    ! FP-CTM
      lonname = 'longitude'
      latname = 'latitude'
      timename = 'Date'
      prsname = 'lev'
    case (5)
    ! NOAA
      lonname = 'longitude'
      latname = 'latitude'
      timename = 'time'
      prsname = 'pressure'     
    case (6)
    ! FP-CTM (monthly)
      lonname = 'longitude'
      latname = 'latitude'
      timename = 'none'
      prsname = 'lev'
  end select
  
  ! from here on FP-CTM daily and monthly treated the same
  if ( iproduct.eq.6 ) iproduct = 4

  print*, 'get_initconc: lonname, latname, prsname =',lonname,latname,prsname


  ! read netcdf file
  ! ----------------

  write(logid,*) 'Reading file: '//trim(filename)

  ! read dimensions
  call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
  call check( nf90_inq_dimid(ncid,trim(lonname),xid) )
  call check( nf90_inquire_dimension(ncid,xid,len=nx) )
  call check( nf90_inq_dimid(ncid,trim(latname),yid) )
  call check( nf90_inquire_dimension(ncid,yid,len=ny) )
  call check( nf90_inq_dimid(ncid,trim(prsname),zid) )
  call check( nf90_inquire_dimension(ncid,zid,len=nz) )
  if ( trim(timename).ne.'none' ) then
    call check( nf90_inq_dimid(ncid,trim(timename),tid) )
    call check( nf90_inquire_dimension(ncid,tid,len=nt) )
  else
    nt = 1
  endif
  if ( ntime.ne.nt ) write(logid,*) 'ERROR get_initconc: inconsistent time dimension'

  ! read dimension variables
  allocate( lon(nx) )
  allocate( lat(ny) )
  call check( nf90_inq_varid(ncid,trim(lonname),varid) )
  call check( nf90_get_var(ncid,varid,lon) )
  call check( nf90_inq_varid(ncid,trim(latname),varid) )
  call check( nf90_get_var(ncid,varid,lat) )

  ! read supporting variables
  select case (iproduct)
    case (1)  
    ! CAMS CH4
      ! hybrid pressure coordinates
      allocate( ap(nz) )
      allocate( bp(nz) )
      call check( nf90_inq_varid(ncid,'hyam',varid) )
      call check( nf90_get_var(ncid,varid,ap) )
      call check( nf90_inq_varid(ncid,'hybm',varid) )
      call check( nf90_get_var(ncid,varid,bp) ) 
      ! read temperature
      allocate( temp(nx,ny,nz,nt) )
      call check( nf90_inq_varid(ncid,'T',varid) )
      call check( nf90_get_var(ncid,varid,temp) )
      call check( nf90_get_att(ncid, varid, 'scale_factor', sclfact) )
      call check( nf90_get_att(ncid, varid, 'add_offset', offset) )
      temp = temp*sclfact + offset
      ! read surface pressure
      allocate( surfp(nx,ny,nt) )
      call check( nf90_inq_varid(ncid,'ps',varid) )
      call check( nf90_get_var(ncid,varid,surfp) )
      call check( nf90_get_att(ncid, varid, 'scale_factor', sclfact) )
      call check( nf90_get_att(ncid, varid, 'add_offset', offset) )
      surfp = surfp*sclfact + offset
      ! mixing ratio is in humid air -> read specific humidity
      allocate( humid(nx,ny,nz,nt) )
      call check( nf90_inq_varid(ncid,'Q',varid) )
      call check( nf90_get_var(ncid,varid,humid) )
      call check( nf90_get_att(ncid, varid, 'scale_factor', sclfact) )
      call check( nf90_get_att(ncid, varid, 'add_offset', offset) )
      humid = humid*sclfact + offset
    case (2)  
    ! EGG4
      ! vertical coordinate is pressure
      allocate( mpres(nz) )
      call check( nf90_inq_varid(ncid,trim(prsname),varid) )
      call check( nf90_get_var(ncid,varid,mpres) )
      ! convert to unit of Pa
      mpres = mpres*100.
    case (3) 
    ! CAMS N2O
      ! vertical coordinate is pressure
      allocate( mpres(nz) )
      call check( nf90_inq_varid(ncid,trim(prsname),varid) )
      call check( nf90_get_var(ncid,varid,mpres) )
    case (4)
    ! FP-CTM
      allocate( mpres(nz) )
      call check( nf90_inq_varid(ncid,trim(prsname),varid) )
      call check( nf90_get_var(ncid,varid,mpres) )
    case (5)
    ! NOAA
      allocate( mpres(nz) )
      call check( nf90_inq_varid(ncid,trim(prsname),varid) )
      call check( nf90_get_var(ncid,varid,mpres) )
  end select

  ! read mixing ratio
  allocate( conc(nx,ny,nz,nt) )
  call check( nf90_inq_varid(ncid,trim(varname),varid) )
  call check( nf90_get_var(ncid,varid,conc(:,:,:,:)) )
  ! check if file contains scalar and offset
  status = nf90_inquire_attribute(ncid,varid,'scale_factor')
  if ( status.eq.nf90_noerr ) then
    call check( nf90_get_att(ncid, varid, 'scale_factor', sclfact) )
    call check( nf90_get_att(ncid, varid, 'add_offset', offset) )
    conc = conc*sclfact + offset
  endif
  call check( nf90_close(ncid) )

  ! adjust mixing ratios
  ! --------------------

  select case ( iproduct )
    case (1)
    ! CAMS CH4
      ! mole fraction of humid air -> convert to dry air
      ! calculate mid layer pressure (mean over nt steps)
      allocate( pres(nx,ny,nz) )
      allocate( pw(nx,ny,nz) )
      do n = 1, nz
        pres(:,:,n) = ap(n) + bp(n)*sum(surfp(:,:,:),dim=3)/real(nt)
      end do
      ! partial pressure water from specific humidity
      pw = sum(humid,dim=4)*pres/real(nt)/(0.622 + 0.378*(sum(humid,dim=4)/real(nt)))
      do n = 1, nt
        conc(:,:,:,n) = conc(:,:,:,n)*pres/(pres - pw)    
      end do
      print*, 'pres = ',pres(nx/2,ny/2,:)
      print*, 'range corr dry air = ',minval(pres/(pres-pw)),maxval(pres/(pres-pw))
      deallocate( humid )
      deallocate( pw )
    case (2)
    ! EGG4
      ! adjust units from mass mixing ratio to mole fraction
      conc = conc*1.e9*mmair/config%molarmass
  end select

  ! calculate geopotential height
  ! -----------------------------

  allocate( geoh(nx,ny,nz) )

  select case (iproduct)
    case (1)
    ! CAMS CH4
      allocate( rpres(nx,ny) )
      allocate( scalh(nx,ny) )
      do n = 1, nz
        rpres = sum(surfp(:,:,:),dim=3)/real(nt)/pres(:,:,n)
        scalh = gasc*sum(temp(:,:,n,:),dim=3)/real(nt)/mmol/gc
        geoh(:,:,n) = scalh*log(rpres)
        print*, 'range(scalh) = ',minval(scalh), maxval(scalh)
      end do
      deallocate( rpres )
      deallocate( scalh )
      deallocate( surfp )
      zkind = 0
    case (2)
    ! EGG4
      do n = 1, nz
        !  at present temp not included so use estimate
        geoh(:,:,n) = (gasc*ts/gc/mmol)*log(psurf/mpres(n))
      end do
      zkind = 0
    case (3)
    ! CAMS N2O
      do n = 1, nz
        ! at present temp not included so use estimate
        geoh(:,:,n) = (gasc*ts/gc/mmol)*log(psurf/mpres(n))
      end do
      zkind = 0
    case (4)
    ! FP-CTM
      ! vertical coordinate is meters above ground 
      do n = 1, nz
        geoh(:,:,n) = mpres(n)
      end do
      zkind = 1
    case (5)
    ! NOAA
      do n = 1, nz
        ! at present temp not included so use estimate
        geoh(:,:,n) = (gasc*ts/gc/mmol)*log(psurf/mpres(n))
      end do
      zkind = 0
  end select

  if ( allocated(mpres) ) deallocate( mpres )
  if ( allocated(pres) )  deallocate( pres )
  if ( allocated(temp) )  deallocate( temp )

  ! adjust lat, lon and alt of grid
  ! -------------------------------

  select case ( iproduct )
    case (1)
    ! CAMS CH4
      midpoints = .true.
    case (2)
    ! EGG4
      ! adjust longitude to be -180 to 180
      where( lon.gt.180. ) lon(:) = lon(:) - 360.
      ! longitude given as eastern boundary
      lon(:) = lon(:) - 0.5*(lon(2) - lon(1))
      ! file contains extra latitude
      ! latitude given from 90 to -90
      do n = 1, ny
        lat(n) = 90. - real(n-1)*(180./real(ny-1)) - 0.5*180./real(ny-1)
      end do
      ny = ny - 1
      midpoints = .true.
    case (3)
    ! CAMS N2O
      ! file contains extra longitude
      nx = nx - 1
      ! latitude given from 90 to -90
      do n = 1, ny
        lat(n) = 90. - real(n)*(180./real(ny))
      end do
      midpoints = .false.
    case (4)
    ! FP-CTM 
      ! file contains extra longitude
      nx = nx - 1
      print*, 'get_initconc: nx = ',nx
      lon(:) = lon(:) + 0.5*(lon(2) - lon(1))
      midpoints = .true.
    case (5)
    ! NOAA
      midpoints = .false.
  end select

  ! sort from south to north and west to east
  print*, 'get_initconc: lon = ',lon
  print*, 'get_iniconc: lat = ',lat
  allocate( indy(ny) )
  allocate( indx(nx) )
  allocate( lato(ny) )
  allocate( lono(nx) )
  allocate( conco(nx,ny,nz,nt) )
  allocate( geoho(nx,ny,nz) )
  lato = dble(lat(1:ny))
  lono = dble(lon(1:nx))
  call sort(ny, lato, indy)
  call sort(nx, lono, indx)
  ! sort from lowest to highest level
  allocate( indz(nz) )
  allocate( alto(nz) )
  alto = dble(geoh(nx/2,ny/2,:))
  call sort(nz, alto, indz)
  print*, 'get_initconc: geoh = ',geoh(nx/2,ny/2,:)
  print*, 'get_initconc: indx = ',indx
  print*, 'get_initconc: indy = ',indy
  print*, 'get_initconc: lono = ',lono
  print*, 'get_initconc: lato = ',lato
  print*, 'get_initconc: indz = ',indz
  conco = conc(indx,indy,indz,:)
  geoho = geoh(indx,indy,indz)
  deallocate( indx )
  deallocate( indy )
  deallocate( lon )
  deallocate( lat )

  ! interpolate to flexpart grid
  ! ----------------------------

  allocate( work1(nx,ny,nz) )
  allocate( work2(nx,ny,nzgrid) )
  allocate( work3(nx,ny,1) )
  allocate( work4(nxgrid,nygrid,1) )

  do n = 1, nt
    work1(:,:,:) = conco(:,:,:,n)
    ! interpolate vertically
    call interpz(nx, ny, nz, zkind, midpoints, sngl(lono), sngl(lato), geoho, work1, work2)
    ! interpolate horizontally
    do kz = 1, nzgrid
      work3(:,:,1) = work2(:,:,kz)
      ! lon_out and lat_out are for global flexpart domain
      call convertgrid(midpoints, nx, ny, sngl(lono), sngl(lato), work3, nx_out, ny_out, lon_out, lat_out, work4)
      concini(:,:,kz,n) = work4(:,:,1)
    end do
  end do

  deallocate( lono )  
  deallocate( lato )
  deallocate( alto )
  deallocate( conc )
  deallocate( geoh )
  deallocate( conco )
  deallocate( geoho )
  deallocate( work1 )
  deallocate( work2 )
  deallocate( work3 )
  deallocate( work4 )

end subroutine get_initconc


