!---------------------------------------------------------------------------------------
! FLEXINVERT: mod_transform
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> mod_transform
!!
!! Purpose:    Module for transforming between the physical and chi space.
!!
!---------------------------------------------------------------------------------------

module mod_transform

  use mod_var
  use mod_states
  use mod_covar
  use mod_settings

  implicit none
  private

  public :: chi2phi, phi2chi, phi2chi_ad, prod_icov

  contains

  ! --------------------------------------------------
  ! chi2phi
  ! --------------------------------------------------
  !> chi2phi
  !!
  !! Purpose:  Converts from chi to physical space.
  !!           phi = B^(1/2)chi + p0
  !!
  ! --------------------------------------------------

  function chi2phi(config, states, covar, chi)

    type(config_t),                  intent(in)  :: config
    type(states_t),                  intent(in)  :: states
    type(covar_t),                   intent(in)  :: covar
    real(kind=8), dimension(nvar,1), intent(in)  :: chi
    real(kind=8), dimension(nvar,1)              :: chi2phi 

    real(kind=8), dimension(neig)                :: tmp
    real(kind=8), dimension(ndvar)               :: tmp1
    integer                                      :: i, j, k, it, jt

    chi2phi(:,1) = 0d0
    do jt = 1, ntstate
      do k = 1, ndvar
        do j = 1, neig
          tmp(j) = covar%evecs(k,j)*sqrt(covar%evals(j))
        end do
        do i = 1, ndvar
          tmp1(i) = dot_product(tmp,covar%evecs(i,:))
        end do
        do it = 1, ntstate
          chi2phi((jt-1)*ndvar+k,1) = chi2phi((jt-1)*ndvar+k,1) + covar%sqcort(jt,it) * &
                                       dot_product(tmp1,chi((it-1)*ndvar+1:it*ndvar,1))
        end do
      end do
    end do
    if ( config%opt_cini ) then
      chi2phi(npvar+1:nvar,1) = chi(npvar+1:nvar,1) * config%cinierr
    endif

    chi2phi(:,1) = chi2phi(:,1) + dble(states%px0)

  end function chi2phi

  ! --------------------------------------------------
  ! phi2chi
  ! --------------------------------------------------
  !> phi2chi
  !!
  !! Purpose:  Converts from physical to chi space.
  !!           chi = B^(-1/2)(p - p0)
  !!
  ! --------------------------------------------------

  function phi2chi(config, states, covar, phi)

    type(config_t),                  intent(in)  :: config
    type(states_t),                  intent(in)  :: states
    type(covar_t),                   intent(in)  :: covar
    real(kind=8), dimension(nvar,1), intent(in)  :: phi
    real(kind=8), dimension(nvar,1)              :: phi2chi

    real(kind=8), dimension(neig)                :: tmp
    real(kind=8), dimension(ndvar)               :: tmp1
    integer                                      :: i, j, k, it, jt

    phi2chi(:,1) = 0d0
    do jt = 1, ntstate
      do k = 1, ndvar
        do j = 1, neig
          tmp(j) = covar%evecs(k,j)*1d0/sqrt(covar%evals(j))
        end do
        do i = 1, ndvar
          tmp1(i) = dot_product(tmp,covar%evecs(i,:))
        end do
        do it = 1, ntstate
          phi2chi((jt-1)*ndvar+k,1) = phi2chi((jt-1)*ndvar+k,1) + covar%isqcort(jt,it) * &
                                       dot_product(tmp1,(phi((it-1)*ndvar+1:it*ndvar,1)-dble(states%px0((it-1)*ndvar+1:it*ndvar))))
        end do
      end do
    end do
    if ( config%opt_cini ) then
      phi2chi(npvar+1:nvar,1) = (phi(npvar+1:nvar,1) - states%px0(npvar+1:nvar)) *  1./config%cinierr
    endif

  end function phi2chi

  ! --------------------------------------------------
  ! phi2chi_ad
  ! --------------------------------------------------
  !> phi2chi_ad
  !!
  !! Purpose:  Converts from adjoint physical space
  !!           to chi space.
  !!           chi_ad = B^(1/2)*phi_ad
  !!           where phi_ad = grad
  !!
  ! --------------------------------------------------

  function phi2chi_ad(config, covar, phi_ad)

    type(config_t),                  intent(in)  :: config
    type(covar_t),                   intent(in)  :: covar
    real(kind=8), dimension(nvar,1), intent(in)  :: phi_ad
    real(kind=8), dimension(nvar,1)              :: phi2chi_ad

    real(kind=8), dimension(neig)                :: tmp
    real(kind=8), dimension(ndvar)               :: tmp1
    integer                                      :: i, j, k, it, jt

    phi2chi_ad(:,1) = 0d0
    do jt = 1, ntstate
      do k = 1, ndvar
        do j = 1, neig
          tmp(j) = covar%evecs(k,j)*sqrt(covar%evals(j))
        end do
        do i = 1, ndvar
          tmp1(i) = dot_product(tmp,covar%evecs(i,:))
        end do
        do it = 1, ntstate
          phi2chi_ad((jt-1)*ndvar+k,1) = phi2chi_ad((jt-1)*ndvar+k,1) + covar%sqcort(jt,it) * &
                                          dot_product(tmp1, phi_ad((it-1)*ndvar+1:it*ndvar,1))
        end do
      end do
    end do
    if ( config%opt_cini ) then
      phi2chi_ad(npvar+1:nvar,1) = phi_ad(npvar+1:nvar,1) * config%cinierr
    endif

  end function phi2chi_ad

  ! --------------------------------------------------
  ! prod_icov
  ! --------------------------------------------------
  !> prod_icov
  !!
  !! Purpose:  Calculates B^(-1)*x
  !!
  ! --------------------------------------------------

  function prod_icov(config, covar, x)

    type (config_t),                 intent(in)  :: config
    type(covar_t),                   intent(in)  :: covar
    real, dimension(nvar),           intent(in)  :: x
    real, dimension(nvar)                        :: prod_icov

    real(kind=8), dimension(neig)                :: tmp
    real(kind=8), dimension(ndvar)               :: tmp1
    integer                                      :: i, j, k, it, jt

    ! B^(-1)*x for flux variables
    prod_icov(:) = 0.
    do jt = 1, ntstate
      do k = 1, ndvar
        do j = 1, neig
          tmp(j) = covar%evecs(k,j)*1d0/covar%evals(j)
        end do
        do i = 1, ndvar
          tmp1(i) = dot_product(tmp,covar%evecs(i,:))
        end do
        do it = 1, ntstate
          prod_icov((jt-1)*ndvar+k) = prod_icov((jt-1)*ndvar+k) + real( covar%icort(jt,it) * &
                                       dot_product(tmp1, dble(x((it-1)*ndvar+1:it*ndvar)) ), kind=4)
        end do
      end do
    end do
    ! B^(-1)*x for initial mixing ratio scalars
    if ( config%opt_cini ) then
      prod_icov(npvar+1:nvar) = x(npvar+1:nvar)* 1./config%cinierr**2
    endif

  end function prod_icov

  ! --------------------------------------------------

end module mod_transform


